﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Locomotive
{
    //Класс, отвечающий за отрисовку 
    public class DrawningLocomotive
    {
        /// Класс-сущность
        public EntityLocomotive Locomotive { get; protected set; }
        /// Левая координата отрисовки локомотива
        protected float _startPosX;
        /// Верхняя координата отрисовки локомотива
        protected float _startPosY;
        /// Ширина окна отрисовки
        private int? _pictureWidth = null;
        /// Высота окна отрисовки
        private int? _pictureHeight = null;
        /// Ширина отрисовки локомотива
        private readonly int _locomotiveWidth = 110;
        /// Высота отрисовки локомотива
        private readonly int _locomotiveHeight = 50;

        /// Инициализация свойств
        public DrawningLocomotive(int speed, float weight, Color bodyColor)
        {
            Locomotive = new EntityLocomotive(speed, weight, bodyColor);
        }

        // Новый конструктор
        protected DrawningLocomotive (int speed, float weight, Color bodyColor, int locomotiveWidth, int locomotiveHeight) 
            : this (speed, weight, bodyColor)
        {
            _locomotiveWidth = locomotiveWidth;
            _locomotiveHeight = locomotiveHeight;
        }

        public void SetBaseColor(Color color)
        {
            if (Locomotive is EntityWarmlyLocomotive)
            {
                Locomotive = (EntityWarmlyLocomotive)Locomotive;
                if (Locomotive is not null)
                {
                    Locomotive = new EntityWarmlyLocomotive(Locomotive.Speed, Locomotive.Weight, color, (Locomotive as EntityWarmlyLocomotive).ExtraColor, (Locomotive as EntityWarmlyLocomotive).Pipe, (Locomotive as EntityWarmlyLocomotive).FuelStorage);
                    return;
                }
            }
            Locomotive = new EntityLocomotive(Locomotive.Speed, Locomotive.Weight, color);
        }

        /// Установка позиции локомотива
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x < 0 || x + _locomotiveWidth >= width)
            {
                return;
            }
            if (y < 0 || y + _locomotiveHeight >= height)
            {
                return;
            }
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }

        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _locomotiveWidth + Locomotive.Step < _pictureWidth)
                    {
                        _startPosX += Locomotive.Step;
                    }
                    else _startPosX = (int)_pictureWidth - _locomotiveWidth;
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Locomotive.Step >= 0)
                    {
                        _startPosX -= Locomotive.Step;
                    }
                    else _startPosX = 0;
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - Locomotive.Step >= 0)
                    {
                        _startPosY -= Locomotive.Step;
                    }
                    else _startPosY = 0;
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _locomotiveHeight + Locomotive.Step < _pictureHeight)
                    {
                        _startPosY += Locomotive.Step;
                    }
                    else _startPosY = (int)_pictureHeight - _locomotiveHeight;
                    break;
            }
        }

        public virtual void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
                    || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);
            //тело
            g.FillRectangle(new SolidBrush(Locomotive?.BodyColor ?? Color.Brown), _startPosX , _startPosY, 110 - 10, 50 - 10);
            //окна
            g.FillRectangle(new SolidBrush(Color.Blue), _startPosX + 10, _startPosY + 10, 10, 10);
            g.FillRectangle(new SolidBrush(Color.Blue), _startPosX + 30, _startPosY + 10, 10, 10);
            g.FillRectangle(new SolidBrush(Color.Blue), _startPosX + 80, _startPosY + 10, 10, 10);
            //дверь
            g.DrawRectangle(pen, _startPosX + 50, _startPosY + 10, 10, 20);
            //колеса
            g.FillEllipse(new SolidBrush(Color.Black), _startPosX, _startPosY + 40, 10, 10);
            g.FillEllipse(new SolidBrush(Color.Black), _startPosX + 20, _startPosY + 40, 10, 10);
            g.FillEllipse(new SolidBrush(Color.Black), _startPosX + 70, _startPosY + 40, 10, 10);
            g.FillEllipse(new SolidBrush(Color.Black), _startPosX + 90, _startPosY + 40, 10, 10);
            //черный прямоугольник
            g.FillRectangle(new SolidBrush(Locomotive?.BodyColor ?? Color.Black), _startPosX + 100, _startPosY + 10, 10, 30);
        }

        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _locomotiveWidth || _pictureHeight <= _locomotiveHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _locomotiveWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _locomotiveWidth;
            }
            if (_startPosY + _locomotiveHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _locomotiveHeight;
            }
        }
        // Получение текущей позиции объекта
        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return (_startPosX, _startPosX + _locomotiveWidth, _startPosY, _startPosY + _locomotiveHeight);
        }
    }
}
